@@ -93,9 +93,9 @@ namespace eval Tree {
         {-height           TkResource "" 0 listbox}
         {-selectfill       Boolean 0  0}
         {-showlines        Boolean 1  0}
-        {-linesfill        TkResource black  0 {listbox -foreground}}
+        {-linesfill        TkResource ""  0 {listbox -foreground}}
         {-linestipple      TkResource ""     0 {label -bitmap}}
-	{-crossfill        TkResource black  0 {listbox -foreground}}
+	{-crossfill        TkResource ""  0 {listbox -foreground}}
         {-redraw           Boolean 1  0}
         {-opencmd          String  "" 0}
         {-closecmd         String  "" 0}
@@ -890,16 +890,19 @@ proc Tree::nodes { path node {first ""} {last ""} } {
     if { ![info exists data($node)] } {
         return -code error "node \"$node\" does not exist"
     }
-
+    set nodes {}
+    set res {}
     if { ![string length $first] } {
-        return [lrange $data($node) 1 end]
-    }
-
-    if { ![string length $last] } {
-        return [lindex [lrange $data($node) 1 end] $first]
+        set nodes [lrange $data($node) 1 end]
+    } elseif { ![string length $last] } {
+        set nodes [lindex [lrange $data($node) 1 end] $first]
     } else {
-        return [lrange [lrange $data($node) 1 end] $first $last]
+        set nodes [lrange [lrange $data($node) 1 end] $first $last]
     }
+    foreach n $nodes {
+        lappend res [_node_name_rev $path $n]
+    }
+    return $res
 }
 
 
@@ -1044,11 +1047,11 @@ proc Tree::edit { path node text {verifycmd ""} {clickres 0} {select 1}} {
                         -selectforeground   [Widget::getoption $path -selectforeground] \
                         -selectbackground   $sbg  \
                         -font               [Widget::getoption $path.$node -font] \
-                        -textvariable       Tree::_edit(text)]
+                        -textvariable       ::Tree::_edit(text)]
         pack $ent -ipadx 8 -anchor w
 
         set idw [$path.c create window $x $y -window $frame -anchor w]
-        trace variable Tree::_edit(text) w \
+        trace add variable _edit(text) write \
 	    [list Tree::_update_edit_size $path $ent $idw $wmax]
         tkwait visibility $ent
         grab  $frame
@@ -1071,14 +1074,14 @@ proc Tree::edit { path node text {verifycmd ""} {clickres 0} {select 1}} {
 
         set ok 0
         while { !$ok } {
-            tkwait variable Tree::_edit(wait)
+            tkwait variable ::Tree::_edit(wait)
             if { !$_edit(wait) || [llength $verifycmd]==0 ||
                  [uplevel \#0 $verifycmd [list $_edit(text)]] } {
                 set ok 1
             }
         }
 
-        trace vdelete Tree::_edit(text) w \
+        trace remove variable _edit(text) write \
 	    [list Tree::_update_edit_size $path $ent $idw $wmax]
         grab release $frame
         BWidget::focus release $ent
@@ -2054,6 +2057,10 @@ proc Tree::_keynav {which win} {
 #	The current node.
 
 proc Tree::_get_current_node {win} {
+    # TCL9 does not resolve sub-namespaces from parents (root) any more.
+    # The following sub-namespace ::Tree::selectTree was not observed
+    # on run-time tests. Nor ::selectTree
+    # So keep this for TCL 9
     if {[info exists selectTree::selectCursor($win)]} {
 	set result $selectTree::selectCursor($win)
     } elseif {[llength [set selList [$win selection get]]]} {
@@ -2215,13 +2222,13 @@ proc Tree::_mouse_select { path cmd args } {
 
 proc Tree::_node_name { path node } {
     # Make sure node names are safe as tags and variable names
-    set map [list & \1 | \2 ^ \3 ! \4 :: \5]
+    set map [list & \1 | \2 ^ \3 ! \4 : \5]
     return  [string map $map $node]
 }
 
 proc Tree::_node_name_rev { path node } {
     # Allow reverse interpretation of node names
-    set map [list \1 & \2 | \3 ^ \4 ! \5 ::]
+    set map [list \1 & \2 | \3 ^ \4 ! \5 :]
     return  [string map $map $node]
 }
 
@@ -2233,6 +2240,10 @@ proc Tree::_destroy { path } {
     variable $path
     upvar 0  $path data
 
+    if { ![info exists data] && [string match ".#BWidget.#Class*" $path] } {
+        # this is a proxy win to query xrdb
+        return
+    }
     if { $data(upd,afterid) != "" } {
         after cancel $data(upd,afterid)
     }
